@@ -24,7 +24,7 @@ from xml.sax.saxutils import escape, unescape
 
 import editabletuple
 
-__version__ = '0.6.0'
+__version__ = '0.7.0'
 
 DATE_SENTINAL = datetime.date(1808, 8, 8)
 DATETIME_SENTINAL = datetime.datetime(1808, 8, 8, 8, 8, 8)
@@ -94,7 +94,7 @@ class Tdb:
         '''Writes this Tdb's tables to a string which is then returned.'''
         out = io.StringIO()
         try:
-            self.dump(out, decimals)
+            self.dump(out, decimals=decimals)
             return out.getvalue()
         finally:
             out.close()
@@ -187,10 +187,18 @@ def _read_records(text, table, lino):
             elif kind == 'real':
                 text, lino = _handle_real(text, record, column, lino)
             else:
-                raise Error(f'{lino}#expected an {kind}')
+                raise Error(f'E100#{lino}:expected an {kind}')
             column += 1
         elif c in '0123456789':
-            if kind == 'int':
+            if kind == 'bool':
+                if (c in '01' and len(text) > 1 and
+                        text[1] not in '.eE0123456789'):
+                    _handle_bool(kind, c == '1', record, column, lino)
+                    text = text[1:]
+                else:
+                    raise Error(
+                        f'E105#{lino}:got {text[:2]} expected a {kind}')
+            elif kind == 'int':
                 text, lino = _handle_int(text, record, column, lino)
             elif kind == 'real':
                 text, lino = _handle_real(text, record, column, lino)
@@ -199,15 +207,15 @@ def _read_records(text, table, lino):
             elif kind == 'datetime':
                 text, lino = _handle_datetime(text, record, column, lino)
             else:
-                raise Error(f'{lino}#expected an {kind}')
+                raise Error(f'E110#{lino}:expected an {kind}')
             column += 1
         elif c == ']': # end of table
             if 0 < column < columns:
-                raise Error(f'{lino}#incomplete record {column + 1}/'
+                raise Error(f'E120#{lino}:incomplete record {column + 1}/'
                             f'{columns} fields')
             return _skip_ws(text[1:], lino)
         else:
-            raise Error(f'{lino}#invalid character {c!r}')
+            raise Error(f'E130#{lino}:invalid character {c!r}')
         if column == columns:
             table.append(record)
             record = None
@@ -222,18 +230,19 @@ def _handle_sentinal(field_meta, record, column, lino):
     sentinal = field_meta.sentinal
     if sentinal is not None:
         return sentinal
-    raise Error(f'{lino}#{field_meta.kind} fields don\'t allow sentinals')
+    raise Error(
+        f'E140#{lino}:{field_meta.kind} fields don\'t allow sentinals')
 
 
 def _handle_bool(kind, value, record, column, lino):
     if kind != 'bool':
-        raise Error(f'{lino}#expected type {kind}, got a bool')
+        raise Error(f'E150#{lino}:expected type {kind}, got a bool')
     record[column] = value
 
 
 def _handle_bytes(kind, text, record, column, lino):
     if kind != 'bytes':
-        raise Error(f'{lino}#expected type {kind}, got a bytes')
+        raise Error(f'E160#{lino}:expected type {kind}, got a bytes')
     found, text, lino = _find(text, ')', 'expected to find ")"', lino)
     record[column] = bytes.fromhex(found)
     return text, lino # skip )
@@ -241,7 +250,7 @@ def _handle_bytes(kind, text, record, column, lino):
 
 def _handle_str(kind, text, record, column, lino):
     if kind != 'str':
-        raise Error(f'{lino}#expected type {kind}, got a str')
+        raise Error(f'E170#{lino}:expected type {kind}, got a str')
     found, text, lino = _find(text, '>', 'expected to find ">"', lino)
     record[column] = unescape(found)
     return text, lino # skip >
@@ -253,7 +262,7 @@ def _handle_int(text, record, column, lino):
         record[column] = int(found)
         return text, lino
     except ValueError as err:
-        raise Error(f'{lino}#invalid int: {found!r}: {err}')
+        raise Error(f'E180#{lino}:invalid int: {found!r}: {err}')
 
 
 def _handle_real(text, record, column, lino):
@@ -262,7 +271,7 @@ def _handle_real(text, record, column, lino):
         record[column] = float(found)
         return text, lino
     except ValueError as err:
-        raise Error(f'{lino}#invalid real: {found!r}: {err}')
+        raise Error(f'E190#{lino}:invalid real: {found!r}: {err}')
 
 
 def _handle_date(text, record, column, lino):
@@ -271,7 +280,7 @@ def _handle_date(text, record, column, lino):
         record[column] = datetime.date.fromisoformat(found)
         return text, lino
     except ValueError as err:
-        raise Error(f'{lino}#invalid date: {found!r}: {err}')
+        raise Error(f'E200#{lino}:invalid date: {found!r}: {err}')
 
 
 def _handle_datetime(text, record, column, lino):
@@ -280,7 +289,7 @@ def _handle_datetime(text, record, column, lino):
         record[column] = datetime.datetime.fromisoformat(found)
         return text, lino
     except ValueError as err:
-        raise Error(f'{lino}#invalid datetime: {found!r}: {err}')
+        raise Error(f'E210#{lino}:invalid datetime: {found!r}: {err}')
 
 
 def _scan(text, valid, lino):
@@ -291,7 +300,7 @@ def _scan(text, valid, lino):
         if c not in valid:
             return text[end:], text[:end], lino
         end += 1
-    raise Error(f'{lino}#unexpected end of data')
+    raise Error(f'E220#{lino}:unexpected end of data')
 
 
 def _skip_ws(text, lino):
@@ -310,7 +319,7 @@ def _skip_ws(text, lino):
 def _find(text, what, message, lino):
     end = text.find(what)
     if end == -1:
-        raise Error(f'{lino}#{message}')
+        raise Error(f'E230#{lino}:{message}')
     lino += text[:end].count('\n')
     return text[:end], text[end + 1:], lino
 
@@ -425,8 +434,38 @@ class Error(Exception):
 if __name__ == '__main__':
     import sys
     if len(sys.argv) == 1 or sys.argv[1] in {'-h', '--help'}:
-        raise SystemExit('usage: tdb.py <infile.tdb> [outfile.tdb]')
-    infile = sys.argv[1]
-    outfile = '-' if len(sys.argv) == 2 else sys.argv[2]
+        raise SystemExit('''\
+usage: tdb.py [-d|--decimals N] <infile.tdb> [outfile.tdb]
+
+-d, --decimals N  0-19; default 0 (use fewest); 1-19 use exactly
+
+Output is to stdout if there's no outfile or outfile is -.
+''')
+    decimals = -1
+    want_decimals = False
+    infile = None
+    outfile = '-'
+    for arg in sys.argv[1:]:
+        if want_decimals:
+            decimals = int(arg)
+            want_decimals = False
+        elif arg in {'-d', '--decimals'}:
+            want_decimals = True
+        elif arg.startswith('-d'):
+            arg = arg[2:]
+            if arg.startswith('='):
+                arg = arg[1:]
+            decimals = int(arg)
+        elif arg.startswith('--decimals'):
+            arg = arg[10:]
+            if arg.startswith('='):
+                arg = arg[1:]
+            decimals = int(arg)
+        elif infile is None:
+            infile = arg
+        else:
+            outfile = arg
+    if not (1 <= decimals <= 19):
+        decimals = -1
     db = load(infile)
-    db.dump(outfile)
+    db.dump(outfile, decimals=decimals)
